/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xpvox.h"

extern FD_xpvox *fdui;

struct sound *lookup_sound(char *name)
{
struct sound *pa;

for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}

return 0;/*not found*/
}/* end function lookup_sound */


struct sound *install_sound_at_end_of_list(char *name)
{
struct sound *plast, *pnew;

if(debug_flag)
	{
	fprintf(stderr,\
	"install_sound_at_end_off_list(): arg name=%s\n", name);
	}

pnew = lookup_sound(name);
if(pnew) return(pnew);/* already there */

/* create new structure */
pnew = (struct sound *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = soundtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !soundtab[0] ) soundtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
soundtab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_sound */


int delete_sound(char *name) /* delete entry from double linked list */
{
struct sound *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stderr, "delete_sound(): arg name=%s\n", name);
	}

if(! name) return 0;

pa = soundtab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify soundtab[0] */
	if(pprev == 0) soundtab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify soundtab[1] */
	if(pnext == 0) soundtab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	/* do not delete pfd! it points to data that may be in use by an other sound (same font) */
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	} /* end for all structures */
} /* end function delete_sound */


int delete_all_sounds() /* delete all entries from table */
{
struct sound *pa;

if(debug_flag)
	{
	fprintf(stderr, "delete_all_sounds() arg none\n");
	}

while(1)
	{	
	pa = soundtab[0];
	if(! pa) break;
	soundtab[0] = pa -> nxtentr;

	free(pa -> name);
	free(pa); /* free structure */
	} /* end while all structures */

soundtab[1] = 0;

return 1;
}/* end function delete_all_sounds */


int load_sounds()
{
int a;
FILE *fileptr;
char temp[READSIZE];
char pathfilename[READSIZE];
struct sound *pa;
extern char *strsave();
char name[READSIZE];
int browser_line; /* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
int status;
int line;
int arg_count;
int play_sequence_number = 0;

if(debug_flag)
	{
	fprintf(stderr, "load_sounds() arg none\n");
	}
	
delete_all_sounds();

sprintf(pathfilename, "%s/.xpvox/sounds.dat", home_dir);
fileptr = fopen(pathfilename, "r");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for read\nadd a sound should create this file", 1);

	if(debug_flag)
		{
		fprintf(stderr, "could not load file %s\n", pathfilename);
		}

	return 0;
	} 

/* for all lines in data_file_name */
line = 0;
while(1)
	{
	a = readline(fileptr, temp);
	if(a == EOF) break;/* empty file or last line */

	arg_count = sscanf(temp,\
	"%s %d %d %d",\
	name,\
	&browser_line,\
	&status,\
	&play_sequence_number);

	if(debug_flag)
		{
		fprintf(stderr, "arg_count=%d\n", arg_count);
		}

	/* increment line counter for error reporting */
	line ++;

	/* increment line counter for error reporting */
	line ++;

	/* create a structure entry */
	pa = install_sound_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stderr,\
			"load_sound: cannot install sound %s\n", temp);
			}
		return 0;
		}

	pa -> status = status;
	pa -> browser_line = browser_line;
	pa -> play_sequence_number = play_sequence_number;

	}/* end while all lines in data_file_name */

sort_sounds();

renumber_sound_list_play_numbers();

return 1;
} /* end function load sound */


int save_sounds()
{
FILE *fileptr;
char pathfilename[READSIZE];
char pathfilename2[READSIZE];
struct sound *pa;
int max_sound;

if(debug_flag)
	{
	fprintf(stderr, "save_sounds() arg none\n");
	}

sprintf(pathfilename, "%s/.xpvox/sounds.tmp", home_dir);
fileptr = fopen(pathfilename, "w");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for write", 1);

	if(debug_flag)
		{
		fprintf(stderr,\
		"could not open file %s for write\n", pathfilename);
		}
	return 0;
	} 

max_sound = 0;
for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(fileptr, "%s %d %d %d\n",\
	pa -> name,\
	pa -> browser_line,\
	pa -> status,\
	pa -> play_sequence_number\
	);

	} /* end while all elements in list */

fclose(fileptr);

/* set some path file names */
sprintf(pathfilename, "%s/.xpvox/sounds.dat", home_dir);

sprintf(pathfilename2, "%s/.xpvox/sounds.dat~", home_dir);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* rename .dat to .dat~ */
if( rename(pathfilename, pathfilename2) == -1)
	{
	if(debug_flag)
		{
		fprintf(stderr, "save_sounds(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	}

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/.xpvox/sounds.tmp", home_dir);
if( rename(pathfilename2, pathfilename) == -1)
	{
	fl_show_alert("could not rename", pathfilename2, pathfilename, 1);

	if(debug_flag)
		{
		fprintf(stderr, "save_sounds(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return 0;
	}

return 1;/* ok */
}/* end function save_sound */


int show_sounds(int position)
{
char temp[READSIZE];
char temp2[READSIZE];
struct sound *pa;
char formatstr[20];
int browser_line;
int browser_topline;
char *strsave();
int maxline;
char *ptr;

if(debug_flag)
	{
	fprintf(stderr, "show_sounds(): arg position=%d\n", position);
	}

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> sound_browser);

fl_freeze_form(fdui -> xpvox);

fl_clear_browser(fdui -> sound_browser);

browser_line = 1;
for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
//	fixed spacing
//	strcpy(formatstr, "@f");

	strcpy(formatstr, "");

	/* black is default color */
	strcat(formatstr, "@C0"); /* black */

	/* colors */

//	strcat(formatstr, "@C17");	/* dark grey */
//	strcat(formatstr, "@C18");	/* dark green */
//	strcat(formatstr, "@C20");	/* dark violet */
//	strcat(formatstr, "@C22");	/* dark brown */
//	strcat(formatstr, "@C24");	/* orange */
//	strcat(formatstr, "@C27");	/* violet */
//	strcat(formatstr, "@C29");	/* dark blue */
//	strcat(formatstr, "@_");	/* underscore */

	/* if to be played in sequence */
	if(pa -> play_sequence_number > 0) strcat(formatstr, "@_");

	if(pa -> status & LEAD_IN) strcat(formatstr, "@C1"); /* red */
	if(pa -> status & LEAD_OUT) strcat(formatstr, "@C20"); /* violet */
	if( (pa -> status & LEAD_IN) && (pa -> status & LEAD_OUT) ) strcat(formatstr, "@C18"); /* dark green */

	/* end colors */

	if(! show_sounds_without_path_flag)
		{
		sprintf(temp2, "%s", pa -> name);
		}
	else
		{
		ptr = strrchr(pa -> name, '/');
		if(ptr)
			{
			strcpy(temp2, ptr + 1);
			}
		else
			{		
			strcpy(temp2, pa -> name);
			}
		}

	if(pa -> play_sequence_number > 0)
		{
		sprintf(temp, "%s %d %s", formatstr, pa -> play_sequence_number, temp2);
		}
	else
		{
		sprintf(temp, "%s %s", formatstr, temp2);
		}

	if(debug_flag)
		{
		fprintf(stderr, "show_sounds(): pa=%p pa->name=%p adding temp=%s\n", pa , pa -> name, temp);
		}

	/*
	xforms only allows 1024 bytes in a browser line,
	should we shift right so the end (filename) is visible?
	*/
	temp[1024] = 0;

	fl_add_browser_line(fdui -> sound_browser, temp);

	pa -> browser_line = browser_line;/* first one is 1 */
	browser_line++;
	}/* end for all elements in list */

/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last sound visible */
	maxline = fl_get_browser_maxline(fdui -> sound_browser);
	fl_set_browser_topline(fdui -> sound_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> sound_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> sound_browser, browser_topline);
	}	
if(position == SHOW_SELECTED)
	{
	fl_set_browser_topline(fdui -> sound_browser, selected_sound - 1);
	fl_select_browser_line(fdui -> sound_browser, selected_sound);
	}	

fl_unfreeze_form(fdui -> xpvox);

return 1;
} /* end function show_sounds */


char *line_to_filename(int line)
{
struct sound *pa;

if(debug_flag)
	{
	fprintf(stderr, "line_to_filename(): arg line=%d\n", line);
	}

if(line < 0) return 0;

for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		return pa -> name;
		}/* end for all entries */
	} /* end for all entries in list */

/* not found */
return 0;
} /* end function line_to_filename */


int add_sound(char *text)
{
struct sound *pa;

if(debug_flag)
	{
	fprintf(stderr, "add_sound(): arg text=%s\n", text);
	}

/* argument check */
if(! text) return 0;

pa = install_sound_at_end_of_list(text);
if(! pa) return 0;

pa -> status = 0;
pa -> status |= NEW_ENTRY;

sort_sounds();

save_sounds();

show_sounds(SHOW_SAME_POSITION);

return 1;
}/* end function add_sound */


void replace(char *text, char *string, char *replace, char *result)
{
while(1)
	{
	if( (char *)strstr(text, string) == text)
		{
		*result = 0;
		strcat(result, replace);
		text += strlen(string);
		result += strlen(replace);
		}
	*result = *text;
	if(*text == 0)break;
	result++;
	text++;
	}
}/* end function replace */


struct sound *install_sound_in_position(char *name, int *new_flag)
{
struct sound *pa, *pprev, *plast, *plastmatch, *pnew, *pnext;
struct sound *lookup_sound();
int have_bigger_flag;

if(debug_flag)
	{
	fprintf(stderr,\
	"install_sound_in_position():\nname=%s\n",\
	name);
	}
	
/* argument check */
if(! name) return 0;

plastmatch = 0;/* for gcc -Wall */

*new_flag = 0;

/*
Test if sound number already exists, if so, overwrite,
If sound number > then the one in the list, look for the next,
if bigger insert before the bigger one, else at end of list.
This quarantees the sound are always sequential. 
*/
have_bigger_flag = 0;
for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(atoi(pa -> name) == atoi(name) )
		{
		/* already there and leave status as is */
		
		return(pa); 
		}
	
	if(have_bigger_flag)
		{
		/* test if to far */
		if(atoi(name) < atoi(pa -> name) ) break;
		} /* end if smaller */

	if(atoi(name) > atoi(pa -> name) )
		{
		have_bigger_flag = 1;
		} /* end if bigger */

	/* save last position */
	plastmatch = pa;
	} /* end for all entries in list */

if(debug_flag)
	{
	fprintf(stderr,\
	"ouside for plastmatch=%p\n", plastmatch);
	}

/* plastmatch now points to insert point, or zero if not found, */

/* create new structure */
pnew = (struct sound *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* flag new structure entry was created */ 
*new_flag = 1;

/* add to end list if no match */
if(! plastmatch)
	{
	/* get previous structure */
	plast = soundtab[1]; /* end list */

	/* set new structure pointers */
	pnew -> nxtentr = 0; /* new points top zero (is end) */
	pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

	/* set previous structure pointers */
	if( !soundtab[0] ) soundtab[0] = pnew; /* first element in list */
	else plast -> nxtentr = pnew;

	/* set array end pointer */
	soundtab[1] = pnew;

	/* return pointer to new structure */
	return pnew;
	}/* end if ! plastmatch */

/* plastmatch now holds insert position, insert sound */
/* insert the structure AFTER plastmatch */
		
/* get previous and next structure */
pnext = plastmatch -> nxtentr;
pprev = plastmatch;

/* set pointers for new struture */
pnew -> nxtentr = pnext;
pnew -> prventr = pprev;

/* set pointers for next structure */
if(pnext == 0) soundtab[1] = pnew;
else pnext -> prventr = pnew;		

/* set pointer for previous structure */
if(pprev == 0) soundtab[0] = pnew;
else pprev -> nxtentr = pnew;
		
/* mark it as a new entry */
pnew -> status = NEW_ENTRY;

/* return pointer to new structure */
return pnew;
}/* end function install_sound_in_position */


int sort_sounds()
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct sound *pa;
struct sound *pb;
int swap_flag;

if(debug_flag)
	{
	fprintf(stderr, "sort_sounds(): arg none\n");
	}

while(1)/* go through list again and again */
	{
	swap_flag = 0;
	for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa -> prventr;
		if(pb)
			{
			if(strcmp(pa -> name, pb -> name) < 0)
				{
				/* indicate position was swapped */
				swap_flag = swap_sound_position(pa , pb);
				}/* end if condition */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_sound */


int swap_sound_position(struct sound *ptop, struct sound *pbottom)
{
struct sound *punder;
struct sound *pabove;

if(debug_flag)
	{
	fprintf(stderr,\
	"swap_position(): swapping top=%p bottom=%p\n", ptop, pbottom);
	}

if(! ptop) return 0;
if(! pbottom) return 0;

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stderr,\
	"swap_position(): punder=%p\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stderr,\
	"swap_position(): pabove=%p\n", pabove);
	}

/* the next pointer in punder (or soundtab[0]) must now point to ptop */
if(! punder)
	{
	soundtab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in soundtabtab */
if(! pabove)
	{
	soundtab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_sound_position */


int set_sound(int position, int mode)
{
struct sound *pa;

if(debug_flag)
	{
	fprintf(stderr, "set_sound(): arg position=%d mode=%d\n", position, mode);
	}

for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(mode == LEAD_IN)
		{
		if(pa -> browser_line == position)
			{
			pa -> status |= LEAD_IN;
			}
		else // reset all others
			{
			pa -> status &= ~LEAD_IN;
			}
		}
	else if(mode == LEAD_OUT)
		{
		if(pa -> browser_line == position)
			{
			pa -> status |= LEAD_OUT;
			}
		else // reset all others
			{
			pa -> status &= ~LEAD_OUT;
			}
		}

	} /* end for all entries in list */

show_sounds(SHOW_SAME_POSITION);

save_sounds();

return 1;
} /* end function set_sound */


int get_sound(int mode, int *sound)
{
struct sound *pa;
char temp[80];

if(debug_flag)
	{
	fprintf(stderr,\
	"get_sound(): arg mode=%d using auto_play_sequence_number=%d\n",\
	mode, auto_play_sequence_number);
	}

if(mode == SELECTED_SOUND)
	{
	*sound = selected_sound;

	return 1;
	}

if(mode == AUTO_PLAY_SOUND)
	{
	if(! auto_play_sequence_flag) return 0;

	pa = find_entry_in_play_sequence(auto_play_sequence_number);
	if(pa)
		{
		*sound = pa -> browser_line;

		fl_set_object_color(fdui -> timed_play_button, FL_COL1, FL_RED);
		fl_set_browser_topline(fdui -> sound_browser, pa -> browser_line - 1);

		while(playing_flag)
			{
			usleep(1000);
			}

		auto_play_sequence_number++;

		sprintf(temp, "%d", auto_play_sequence_number);
		fl_set_input(fdui -> play_one_per_day_next_input_field, temp);

		/* if next entry needs to be played next day, reset the flag set by the timer */
		if(play_one_per_day_flag)
			{
			auto_play_sequence_flag = 0;
			}

		/* save auto_play_sequence_number */
		save_audio_settings();		

		return 1;
		}
	else // no more entries
		{
		auto_transmit_flag = 0;
		auto_play_sequence_flag = 0;  
		auto_play_sequence_number = 1;

		fl_set_object_color(fdui -> timed_play_button, FL_YELLOW, FL_DARKORANGE);
		fl_set_button(fdui -> timed_play_button, 0);

		sprintf(temp, "%d", auto_play_sequence_number);
		fl_set_input(fdui -> play_one_per_day_next_input_field, temp);

		save_audio_settings();

		return 0;
		}

	}

/* find the entry with LEAD_IN or LEAD_OUT set, in the linked list */
for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	switch(mode)
		{
		case LEAD_IN_SOUND:
			if(pa -> status & LEAD_IN)
				{
				*sound = pa -> browser_line;

				return 1;
				}
			break;
		case LEAD_OUT_SOUND:
			if(pa -> status & LEAD_OUT)
				{
				*sound = pa -> browser_line;

				return 1;
				}
			break;
		} /* end switch mode */

	} /* end for all entries in list */

/* not found */
return 0;
} /* end function get_sound */


int get_next_in_play_sequence(int *next_in_sequence)
{
struct sound *pa;
int max_sequence_number_found;

if(debug_flag)
	{
	fprintf(stderr, "get_next_in_play_sequence(): arg none\n");
	}

max_sequence_number_found = -1;
for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> play_sequence_number > max_sequence_number_found)
		{
		max_sequence_number_found = pa -> play_sequence_number;
		}
	} /* end for all entries in list */

max_sequence_number_found++;

*next_in_sequence = max_sequence_number_found;

return 1;
} /* end function get_next_in_play_sequence */
 

int set_play_sequence_number(int browser_line, int nr)
{
struct sound *pa;

if(debug_flag)
	{
	fprintf(stderr, "set_play_sequence_number(): arg  browser_line=%d nr=%d\n", browser_line, nr);
	}

for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == browser_line)
		{
		pa -> play_sequence_number = nr;

		return 1;
		}
	} /* end for all entries in list */

if(debug_flag)
	{
	fprintf(stderr, "set_play_sequence_number(): return 0 not found\n");
	}

return 0; // not found
} /* end function set_play_sequence_number */


int add_play_sequence_number(int browser_line, int nr)
{
struct sound *pa;
int next_free = 1;
int exists_flag = 0;

if(debug_flag)
	{
	fprintf(stderr, "add_play_sequence_number(): arg browser_line=%d nr=%d\n", browser_line, nr);
	}

if(browser_line < 0) return 0;
if(nr < 1) nr = 0;

if(! get_next_in_play_sequence(&next_free) ) return 0;
if(nr > next_free)
	{
	nr = next_free;
	if(! set_play_sequence_number(browser_line, nr) ) return 0;

	return 1;
	}

/* test if nr exists */
exists_flag = 0;
for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> play_sequence_number == nr)
		{
		exists_flag = 1;

		break;			
		}
	}

if(exists_flag)
	{
	if(debug_flag)
		{
		fprintf(stderr, "add_play_sequence_number() nr=%d exists, doing copy up\n", nr);
		}

	/* copy up list from nr */
	for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
		{
		if(pa -> play_sequence_number >= 1)
			{
			if(pa -> play_sequence_number >= nr) pa -> play_sequence_number++;	
			}
		}
	}

if(! set_play_sequence_number(browser_line, nr) ) return 0;

/* renumber list */
if(! renumber_sound_list_play_numbers() ) return 0;

return 1;
} /* end function add_play_sequence_number */


int renumber_sound_list_play_numbers()
{
int i;
struct sound *pa;
int next_number;
int sequence_number;

if(debug_flag)
	{
	fprintf(stderr, "renumber_sound_list_play_numbers(): arg none\n");
	}

/* get next number */
if(! get_next_in_play_sequence(&next_number) ) return 0; 

/* go through list again and again */
sequence_number = 1;
for(i = 1; i < next_number; i++)
	{
	pa = find_entry_in_play_sequence(i);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stderr, "renumber_sound_list_play_numbers() no entry %d found\n", i);
			}

		continue;
		}

	/* modify this number so it is in sequence */
	if(pa -> play_sequence_number > sequence_number)
		{
		if(debug_flag)
			{
			fprintf(stderr,\
			"renumber_sound_list_play_numbers(): next entry in play sequence=%d setting to %d\n",\
			pa -> play_sequence_number, i);
			}

		pa -> play_sequence_number = sequence_number;
		}

	sequence_number++;
	} /* end for all numbers */

return 1;
} /* end function renumber_sound_list_play_numbers */


struct sound *find_entry_in_play_sequence(int nr)
{
struct sound *pa;

if(debug_flag)
	{
	fprintf(stderr, "find_entry_in_play_sequence(): arg nr=%d\n", nr);
	}

if(nr < 0) return 0;

for(pa = soundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> play_sequence_number == nr)
		{
		return pa;
		}
	} /* end for all entries in list */

return 0; // not found 
} /* end function find_entry_in_play_sequence */


